package cz.upol.zp4jv.lecture02;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.net.ServerSocket;
import java.net.Socket;

/**
 * Vicevlaknova varianta ServerExample
 */
public class MTServerExample {
	
	private static boolean stopServer = false;
	
	public static void main(String[] args) throws IOException {
		ServerSocket srvSocket = new ServerSocket(4242);
		
		while (!isStopServer()) {
			System.out.println("Waiting for a client");
			Socket clientSocket = srvSocket.accept();
			Thread serverThread = new ServerThread(clientSocket);
			serverThread.start();
		}
		System.out.println("So long!");
		srvSocket.close();
	}
	
	
	/**
	 * Vlakno komunikujici s klientem
	 */
	private static class ServerThread extends Thread {
		
		private Socket clientSocket;

		public ServerThread(Socket clientSocket) {
			this.clientSocket = clientSocket;
		}
		
		public void run() {
			try (BufferedReader rd = new BufferedReader(new InputStreamReader(clientSocket.getInputStream()));
			     BufferedWriter wr = new BufferedWriter(new OutputStreamWriter(clientSocket.getOutputStream()))) {
				
				processRequests(rd, wr);
			} catch (IOException e) {
				e.printStackTrace();
			} finally {
				try {
					clientSocket.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}
	}
	
	/**
	 * Metoda zajistujici komunikaci s klientem pres dvojici objektu tridy Reader+Writer
	 */
	private static void processRequests(BufferedReader rd, BufferedWriter wr) throws IOException {
		while (true) {
			String line = rd.readLine();
			if (line == null) break;
			
			System.out.println("Request: " + line);
			if (line.equalsIgnoreCase("quit")) break;
			if (line.equalsIgnoreCase("stopServer")) {
				setStopServer(true);
				break;
			}
			
			String resp = line.toUpperCase();
			System.out.println("Response: " + resp);
			wr.write(resp);
			wr.write('\n');
			wr.flush();			
		}
	}
	
	
	private static synchronized void setStopServer(boolean value) {
		stopServer = value;
	}
	
	private static synchronized boolean isStopServer() {
		return stopServer;
	}
}
